@@ -1,16 +1,24 @@
+//! Temporal-specific parsing and AST-building utilities for Duper.
+
 use std::borrow::Cow;
 
 use chumsky::prelude::*;
 
 use crate::{
-    DuperTemporal, DuperTemporalInner,
+    ast::{
+        DuperTemporal, DuperTemporalDuration, DuperTemporalInstant, DuperTemporalPlainDate,
+        DuperTemporalPlainDateTime, DuperTemporalPlainMonthDay, DuperTemporalPlainTime,
+        DuperTemporalPlainYearMonth, DuperTemporalUnspecified, DuperTemporalZonedDateTime,
+        DuperValue,
+    },
     parser::{ascii_alphabetic, ascii_alphanumeric, whitespace_and_comments},
 };
 
 // Duper Temporal values
 
-pub(crate) fn temporal_specified<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a known Temporal value.
+pub fn temporal_specified<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     choice((
         temporal_instant(),
         temporal_zoned_date_time(),
@@ -24,8 +32,9 @@ pub(crate) fn temporal_specified<'a>()
     .padded_by(whitespace_and_comments())
 }
 
-pub(crate) fn temporal_instant<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a Temporal Instant, including the identifier and single quotes.
+pub fn temporal_instant<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     just("Instant")
         .padded_by(whitespace_and_comments())
         .ignore_then(just('('))
@@ -39,11 +48,16 @@ pub(crate) fn temporal_instant<'a>()
                 .padded_by(whitespace_and_comments()),
         )
         .then_ignore(just(')'))
-        .map(|instant| DuperTemporal::Instant(DuperTemporalInner(Cow::Borrowed(instant))))
+        .map(|instant| {
+            DuperValue::Temporal(DuperTemporal::Instant {
+                inner: DuperTemporalInstant(Cow::Borrowed(instant)),
+            })
+        })
 }
 
-pub(crate) fn temporal_zoned_date_time<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a Temporal ZonedDateTime, including the identifier and single quotes.
+pub fn temporal_zoned_date_time<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     just("ZonedDateTime")
         .padded_by(whitespace_and_comments())
         .ignore_then(just('('))
@@ -57,11 +71,16 @@ pub(crate) fn temporal_zoned_date_time<'a>()
                 .padded_by(whitespace_and_comments()),
         )
         .then_ignore(just(')'))
-        .map(|instant| DuperTemporal::ZonedDateTime(DuperTemporalInner(Cow::Borrowed(instant))))
+        .map(|instant| {
+            DuperValue::Temporal(DuperTemporal::ZonedDateTime {
+                inner: DuperTemporalZonedDateTime(Cow::Borrowed(instant)),
+            })
+        })
 }
 
-pub(crate) fn temporal_plain_date<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a Temporal PlainDate, including the identifier and single quotes.
+pub fn temporal_plain_date<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     just("PlainDate")
         .padded_by(whitespace_and_comments())
         .ignore_then(just('('))
@@ -75,11 +94,16 @@ pub(crate) fn temporal_plain_date<'a>()
                 .padded_by(whitespace_and_comments()),
         )
         .then_ignore(just(')'))
-        .map(|instant| DuperTemporal::PlainDate(DuperTemporalInner(Cow::Borrowed(instant))))
+        .map(|instant| {
+            DuperValue::Temporal(DuperTemporal::PlainDate {
+                inner: DuperTemporalPlainDate(Cow::Borrowed(instant)),
+            })
+        })
 }
 
-pub(crate) fn temporal_plain_time<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a Temporal PlainTime, including the identifier and single quotes.
+pub fn temporal_plain_time<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     just("PlainTime")
         .padded_by(whitespace_and_comments())
         .ignore_then(just('('))
@@ -93,11 +117,16 @@ pub(crate) fn temporal_plain_time<'a>()
                 .padded_by(whitespace_and_comments()),
         )
         .then_ignore(just(')'))
-        .map(|instant| DuperTemporal::PlainTime(DuperTemporalInner(Cow::Borrowed(instant))))
+        .map(|instant| {
+            DuperValue::Temporal(DuperTemporal::PlainTime {
+                inner: DuperTemporalPlainTime(Cow::Borrowed(instant)),
+            })
+        })
 }
 
-pub(crate) fn temporal_plain_date_time<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a Temporal PlainDateTime, including the identifier and single quotes.
+pub fn temporal_plain_date_time<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     just("PlainDateTime")
         .padded_by(whitespace_and_comments())
         .ignore_then(just('('))
@@ -111,11 +140,16 @@ pub(crate) fn temporal_plain_date_time<'a>()
                 .padded_by(whitespace_and_comments()),
         )
         .then_ignore(just(')'))
-        .map(|instant| DuperTemporal::PlainDateTime(DuperTemporalInner(Cow::Borrowed(instant))))
+        .map(|instant| {
+            DuperValue::Temporal(DuperTemporal::PlainDateTime {
+                inner: DuperTemporalPlainDateTime(Cow::Borrowed(instant)),
+            })
+        })
 }
 
-pub(crate) fn temporal_plain_year_month<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a Temporal PlainYearMonth, including the identifier and single quotes.
+pub fn temporal_plain_year_month<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     just("PlainYearMonth")
         .padded_by(whitespace_and_comments())
         .ignore_then(just('('))
@@ -129,11 +163,16 @@ pub(crate) fn temporal_plain_year_month<'a>()
                 .padded_by(whitespace_and_comments()),
         )
         .then_ignore(just(')'))
-        .map(|instant| DuperTemporal::PlainYearMonth(DuperTemporalInner(Cow::Borrowed(instant))))
+        .map(|instant| {
+            DuperValue::Temporal(DuperTemporal::PlainYearMonth {
+                inner: DuperTemporalPlainYearMonth(Cow::Borrowed(instant)),
+            })
+        })
 }
 
-pub(crate) fn temporal_plain_month_day<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a Temporal PlainMonthDay, including the identifier and single quotes.
+pub fn temporal_plain_month_day<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     just("PlainMonthDay")
         .padded_by(whitespace_and_comments())
         .ignore_then(just('('))
@@ -147,11 +186,16 @@ pub(crate) fn temporal_plain_month_day<'a>()
                 .padded_by(whitespace_and_comments()),
         )
         .then_ignore(just(')'))
-        .map(|instant| DuperTemporal::PlainMonthDay(DuperTemporalInner(Cow::Borrowed(instant))))
+        .map(|instant| {
+            DuperValue::Temporal(DuperTemporal::PlainMonthDay {
+                inner: DuperTemporalPlainMonthDay(Cow::Borrowed(instant)),
+            })
+        })
 }
 
-pub(crate) fn temporal_duration<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a Temporal Duration, including the identifier and single quotes.
+pub fn temporal_duration<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     just("Duration")
         .padded_by(whitespace_and_comments())
         .ignore_then(just('('))
@@ -165,24 +209,32 @@ pub(crate) fn temporal_duration<'a>()
                 .padded_by(whitespace_and_comments()),
         )
         .then_ignore(just(')'))
-        .map(|duration| DuperTemporal::Duration(DuperTemporalInner(Cow::Borrowed(duration))))
+        .map(|duration| {
+            DuperValue::Temporal(DuperTemporal::Duration {
+                inner: DuperTemporalDuration(Cow::Borrowed(duration)),
+            })
+        })
 }
 
-pub(crate) fn temporal_unspecified<'a>()
--> impl Parser<'a, &'a str, DuperTemporal<'a>, extra::Err<Rich<'a, char>>> + Clone {
+/// Parse an unspecified Temporal value, delimited by single quotes.
+pub fn temporal_unspecified<'a>()
+-> impl Parser<'a, &'a str, DuperValue<'a>, extra::Err<Rich<'a, char>>> + Clone {
     unspecified()
         .to_slice()
         .delimited_by(just('\''), just('\''))
         .padded_by(whitespace_and_comments())
         .map(|unspecified| {
-            DuperTemporal::Unspecified(DuperTemporalInner(Cow::Borrowed(unspecified)))
+            DuperValue::Temporal(DuperTemporal::Unspecified {
+                identifier: None,
+                inner: DuperTemporalUnspecified(Cow::Borrowed(unspecified)),
+            })
         })
 }
 
 // Inner values
 
-pub(crate) fn zoned_date_time<'a>()
--> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a ZonedDateTime.
+pub fn zoned_date_time<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     date_time()
         .then(time_offset())
         .then(timezone())
@@ -191,7 +243,8 @@ pub(crate) fn zoned_date_time<'a>()
         .ignored()
 }
 
-pub(crate) fn non_z_zoned_date_time<'a>()
+/// Parse a ZonedDateTime with a non-Z offset.
+pub fn non_z_zoned_date_time<'a>()
 -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     date_time()
         .then(time_num_offset())
@@ -201,45 +254,48 @@ pub(crate) fn non_z_zoned_date_time<'a>()
         .ignored()
 }
 
-pub(crate) fn instant<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
+/// Parse an Instant.
+pub fn instant<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     choice((
         zoned_date_time(),
         date_time().then(time_offset()).padded().ignored(),
     ))
 }
 
-pub(crate) fn non_z_instant<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone
-{
+/// Parse an instant with a non-Z offset.
+pub fn non_z_instant<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     choice((
         non_z_zoned_date_time(),
         date_time().then(time_num_offset()).padded().ignored(),
     ))
 }
 
-pub(crate) fn plain_date<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a PlainDate.
+pub fn plain_date<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     choice((
         plain_date_time(),
         date().then(suffix_tag().repeated()).padded().ignored(),
     ))
 }
 
-pub(crate) fn plain_time<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a PlainTime.
+pub fn plain_time<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     choice((
         plain_date_time(),
         time().then(suffix_tag().repeated()).padded().ignored(),
     ))
 }
 
-pub(crate) fn plain_date_time<'a>()
--> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a PlainDateTime.
+pub fn plain_date_time<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     choice((
         non_z_instant(),
         date_time().then(suffix_tag().repeated()).padded().ignored(),
     ))
 }
 
-pub(crate) fn plain_year_month<'a>()
--> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a PlainYearMonth.
+pub fn plain_year_month<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     choice((
         plain_date(),
         year_month()
@@ -249,15 +305,16 @@ pub(crate) fn plain_year_month<'a>()
     ))
 }
 
-pub(crate) fn plain_month_day<'a>()
--> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a PlainMonthDay.
+pub fn plain_month_day<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     choice((
         plain_date(),
         month_day().then(suffix_tag().repeated()).padded().ignored(),
     ))
 }
 
-pub(crate) fn duration<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
+/// Parse a Duration.
+pub fn duration<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     let fractional = text::int(10).then(just('.').then(text::digits(10).at_most(9)).or_not());
 
     let duration_time = one_of("Tt").then(choice((
@@ -320,8 +377,8 @@ pub(crate) fn duration<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a,
         .ignored()
 }
 
-pub(crate) fn unspecified<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone
-{
+/// Parse an unspecified Temporal value.
+pub fn unspecified<'a>() -> impl Parser<'a, &'a str, (), extra::Err<Rich<'a, char>>> + Clone {
     choice((
         instant(),
         plain_year_month(),
